/***************************************************************************
    begin                : Mon Jul  2 23:00:58 COT 2001
    copyright            : (C) 2001 by Vctor Javier Cerquera, Alejandro Gutirrez
    email                : <javier@gaia.univalle.edu.co>
    											 <alejandro@calipso.com.co>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*	Control es el manejador central de toda la aplicacin.

		Sobre este objeto, recaen las responsabilidades de enlazar mensajes de un
		objeto a primsGLotro, adems de que es el que se encarga de decidir que se debe
		de pintar en un momento dado.

		Este objeto es el nico que sabe que es lo que se est desplegando en cada
		glApplet.

		Esta clase debe ser una experta en seales y slots para poder enlazar las
		restantes; en toda la aplicacin, esta es la nica clase que puede crear
		conexiones entre las seales y los slots.
*/

#ifndef CONTROL_H
#define CONTROL_H

#include "interfaz.h"
#include "mundo.h"
#include "defines.h"
#include "primPrisma.h"
#include "primToro.h"
#include "primEsfera.h"
#include "primDisco.h"
#include "primHyper.h"
#include "primCilindro.h"
#include "primCono.h"
#include "dialogo_seleccion.h"
#include "dialogo_mover.h"
#include "dialogo_rotar.h"
#include "dialogo_escalar.h"
#include "dialogo_recorte.h"
#include "dialogo_mirror.h"
#include "dialogo_render_imp.h"
#include "dialogo_rendergl.h"
#include "dialogo_material.h"
#include "dialogo_about.h"

class control : public QObject
{
  Q_OBJECT

	public:

		control( QObject* parent, const char* name );
	  ~control();
    interfaz * getInterfaz( void );
	  GLuint mundoGL, primsGL;
	
	public slots:

		// Slots relacionados con los glApplets

		void actualizarRIB( void );
    void construirListaPrims( void );
		// Slots relacionados con TODOS los objetos.

		// Slots relacionados con la interfaz, las primitivas y el mundo

 		void dialogo_crear( int );
 		void dialogoSeleccion( void );
 		void mostrarDialogoSeleccion( void );
 		void primCreada( void );
 		void primNoCreada( void );
 		
 	
		void actualizarSeleccion( void );
    void revisarNombre( const QString & );
    void validarNombres( QString );
		// Slots relacionados con el mundo

  	void interpolar( void );
  	void interpolarFin( void );
    void actualizarPrims( void );
    void iniciar( void );
    void record( void );
    void mover( void );
    void rotar( void );
    void escalar( void );
    void recorte( void );
    void mirror( void );
    void material( void );
    void addLista( QString );
    void moverPreview( void );
    void moverTransf( void );
    void calcularMatrizMover( void );
    void rotarPreview( void );
    void rotarTransf( void );
    void calcularMatrizRotar( void );
    void escalarPreview( void );
    void escalarTransf( void );
    void calcularMatrizEscalar( void );
    void recortePreview( void );
    void recorteTransf( void );
    void calcularMatrizrecorte( void );
    void mirrorPreview( void );
    void mirrorTransf( void );
    void calcularMatrizMirror( void );
    void render( void );
    void renderGL( void );
    void generarArchivo( void );
    void ejecutarAnim( void );
    void animar( void );
    void view_foto( void );
    void fov( GLdouble );
    void abrir_escena( void );	
		void nueva_escena( void );	
		void guardar_escena( void );
		void salvarArchivo( void );
    void ultKeyFrame( uint valor ) { ultKeyFrameN = valor;}	
    void actBKColor( QColor * color ){ backgroundC = color; }
    void generarXRIB( void );
    void ensamblarXRIB( void );
    void genEncabezadoRIB( void );
    void guardar_escena_rib( void );
	protected:

	private:

		VectorP * prim;
		VectorC * cadsR;
		primitiva * tempP;
		QString * tempC;
		interfaz * ventPpal;
		uint primOK;
		uint nombOK;
		uint RIBOk;
		uint encabOK;
		uint ensambOK;
		ofstream * salidaRIB;
		mundo *world;
		QStringList * lista;
		dialogo_render_imp * dialRen;
		dialogo_rendergl * dialRenGL;
    dialogo_seleccion * dialSel;
		dialogo_mover * dialMov;
		dialogo_rotar * dialRot;
		dialogo_escalar * dialEsc;
		dialogo_recorte * dialRec;
		dialogo_mirror * dialMir;
    QString * nomValidacion;
    QString * luz;
    Matrix * matT;
		bool modoGL;
		QString * cadEx;
		uint interp;
		
		// Variables de salida RIB
		
		uint ultKeyFrameN;
		uint anchoRIB;
		uint altoRIB;
		uint frameIniRIB;
		uint frameFinRIB;
		uint fpsRIB;
		float detailRIB;
		bool cerrarRIB;
		uint estiloRIB;
		uint xsamplesRIB;
		uint ysamplesRIB;
		uint scanlinesRIB;
		bool cropRIB;
		float xMinCropRIB;
		float yMinCropRIB;
		float xMaxCropRIB;
		float yMaxCropRIB;
		bool radRIB;
		uint radStepsRIB;
		uint radSamplesRIB;
		bool saveRIB;
		QString * fileRIB;
		char cadena[512];
		char tempCad[512];
		bool salvando;
		uint primAnimOk;
		bool animando;
		uint animAct;
		GLdouble fovD;
		uint lastPrim;
		QColor * backgroundC;
		
	signals:
  	
		void _material( QString * );
		
	  void _dialogo_crear_ambiental( void );
	  void _dialogo_crear_omni( void );
	  void _dialogo_crear_spot( void );
  	void _generarInterCreac( void );
  	void _rePaint( void );
  	void _actualizarGL( void );
  	void _actualizarRIB( void );
  	void _camera_edit( void );
  	void _interpolar( void );
  		
	  void _dialogo_crear_ambiental( float, float, float, float );
	  void _dialogo_crear_omni( float, float, float, float, float, float, float );
	  void _dialogo_crear_spot( float, float, float, float, float, float, float, float, float, float, float, float, float );

  	void _baseGL( GLuint );
  	void _cambiarAFrame( int );
  	void _cambiarModoVisual( int, int );
  	void _cambiarCamara( int, int );
  	void _record( void );
  	void _record( uint, float, float, float, float, float, float );
  	void _record( Matrix, QString *, uint );
  	void _getNombres( void );
  	void _getNombresValidar( void );
  	void _nombresOK( void );
  	void _seleccionar( QString, bool );
  	void _inicializar( void );
  	void _resetPreview( void );
  	void _moverPreview( Matrix );
  	void _moverTransf( Matrix );
  	void _rotarPreview( Matrix );
  	void _rotarTransf( Matrix );
  	void _escalarPreview( Matrix );
  	void _escalarTransf( Matrix );
  	void _recortePreview( Matrix );
  	void _recorteTransf( Matrix );
  	void _mirrorPreview( Matrix );
  	void _mirrorTransf( Matrix );
  	void _genRIBFrame( uint );  	
  	void _finGenArch( void );
  	void _finGenArchSalvar( void );
  	void _obtEncabezadoXRib( void );
  	void _obtFramesXRib( void );
};

#endif
