/***************************************************************************
    begin                : Mon Jul  2 23:00:58 COT 2001
    copyright            : (C) 2001 by Vctor Javier Cerquera, Alejandro Gutirrez
    email                : <javier@gaia.univalle.edu.co>
    											 <alejandro@calipso.com.co>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PRIMESFERA_H
#define PRIMESFERA_H

#include "defines.h"
#include "primitiva.h"
#include "colorform_imp.h"
#include "spinboxf.h"
#include "dialogo_crear_esfera.h"

class primEsfera : public primitiva
{
  Q_OBJECT

	public:

		primEsfera( QObject* parent, const char* name, int numFrames, GLuint Lista, QString * cadRib )
							: primitiva( parent, name, numFrames, cadRib )
		{			
			lista = Lista;

			R = 5.0;
			Zmin = -5.0;
			Zmax = 5.0;
			The = 360.0;	
			iterIn = OPTSPHER - 1;
			iterEx = OPTSPHER - 1;
			factIn = (iterIn - 1)/ 360.0;
			factEx = (iterEx - 1)/ 360.0;
			
		
			for(int i = 0, h = 0; i <= iterEx; ++i)
			{
			  for(h = 0; h <= iterIn; ++h)
			  {
			    _puntos1[i][h] = new Matrix( 4, 1 );
			    _puntos2[i][h] = new Matrix( 4, 1 );
			    _puntos3[i][h] = new Matrix( 4, 1 );
			    _puntos4[i][h] = new Matrix( 4, 1 );
			    _puntos5[i][h] = new Matrix( 4, 1 );
			    _puntos6[i][h] = new Matrix( 4, 1 );
			    _puntos7[i][h] = new Matrix( 4, 1 );
			    _puntos8[i][h] = new Matrix( 4, 1 );
			  }
			}
			
			this -> recalcularValoresIniciales( );
			
		}

		
		primEsfera( QObject* parent, const char* name, int numFrames, GLuint Lista, QString * cadRib, QColor *Color, float Radio, float ZMIN, float ZMAX, float theta, QString * mater )
							: primitiva( parent, name, numFrames, cadRib )
		{			
			lista = Lista;

			R = Radio;
			Zmin = ZMIN;
			Zmax = ZMAX;
			The = theta;	
			iterIn = OPTSPHER - 1;
			iterEx = OPTSPHER - 1;
			factIn = (iterIn - 1)/ 360.0;
			factEx = (iterEx - 1)/ 360.0;
			color = Color;
		
			for(int i = 0, h = 0; i <= iterEx; ++i)
			{
			  for(h = 0; h <= iterIn; ++h)
			  {
			    _puntos1[i][h] = new Matrix( 4, 1 );
			    _puntos2[i][h] = new Matrix( 4, 1 );
			    _puntos3[i][h] = new Matrix( 4, 1 );
			    _puntos4[i][h] = new Matrix( 4, 1 );
			    _puntos5[i][h] = new Matrix( 4, 1 );
			    _puntos6[i][h] = new Matrix( 4, 1 );
			    _puntos7[i][h] = new Matrix( 4, 1 );
			    _puntos8[i][h] = new Matrix( 4, 1 );
			  }
			}
			
			matRib = mater;
			
			this -> recalcularValoresIniciales( );
			
		}
			
		~primEsfera()
		{
		  for(int i = 0, h = 0; i <= iterEx; ++i)
			{
	  		for(h = 0; h <= iterIn; ++h)
			  {
			    delete _puntos1[i][h];
			    delete _puntos2[i][h];
			    delete _puntos3[i][h];
			    delete _puntos4[i][h];
			    delete _puntos5[i][h];
			    delete _puntos6[i][h];
			    delete _puntos7[i][h];
			    delete _puntos8[i][h];
	  		}
			}
		}

	public slots:
    void generarCodGL( void ); // Listo
		void generarCodRIB( void ); // Listo
		void generarInterCreac( void );
		void actualizarValores( void );
	  void recalcularValoresIniciales( void );
    void validarNombre( void ) { if( ! dialogo ) emit _validarNombre( *nombre ); }
    void encabezadoXRIB( void );
    void genFrameXRib( void );
	
	protected:

	  dialogo_crear_esfera * dialogo;
		
		int iterIn, iterEx, indIn, indIn2, indEx;		

		float R, Zmax, Zmin, The, factIn, factEx;	
				
		pMatrix _puntos1[OPTSPHER][OPTSPHER];
		pMatrix _puntos2[OPTSPHER][OPTSPHER];
		pMatrix _puntos3[OPTSPHER][OPTSPHER];
		pMatrix _puntos4[OPTSPHER][OPTSPHER];
		pMatrix _puntos5[OPTSPHER][OPTSPHER];
		pMatrix _puntos6[OPTSPHER][OPTSPHER];
		pMatrix _puntos7[OPTSPHER][OPTSPHER];
		pMatrix _puntos8[OPTSPHER][OPTSPHER];

};
#endif
