/***************************************************************************
    begin                : Mon Jul  2 23:00:58 COT 2001
    copyright            : (C) 2001 by Vctor Javier Cerquera, Alejandro Gutirrez
    email                : <javier@gaia.univalle.edu.co>
    											 <alejandro@calipso.com.co>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PRIMTORO_H
#define PRIMTORO_H

#include "defines.h"
#include "primitiva.h"
#include "colorform_imp.h"
#include "spinboxf.h"
#include "dialogo_crear_toro.h"

class primToro : public primitiva
{
  Q_OBJECT

	public:

		primToro( QObject* parent, const char* name, int numFrames, GLuint Lista, QString * cadRib )
							: primitiva( parent, name, numFrames, cadRib )
		{			
			lista = Lista;

			Rm = 1.0;
			Ry = 5.0;
			M = 360.0;
			N1 = 0.0;
			N2 = 360.0;	
			iterIn = OPTTORUS;
			iterEx = OPTTORUS;
			factIn = (iterIn - 2)/ 360.0;
			factEx = (iterEx - 2)/ 360.0;
			
		
			for(int i = 0, h = 0; i < iterEx; ++i)
			{
			  for(h = 0; h < iterIn; ++h)
			  {
			    _puntos[i][h] = new Matrix( 4, 1 );
			  }
			}
			
			this -> recalcularValoresIniciales( );
			
		}
	
		
		primToro( QObject* parent, const char* name, int numFrames, GLuint Lista, QString * cadRib, QColor *Color, float RMIN, float RMAY, float PHIMIN, float PHIMAX, float theta, QString * mater )
							: primitiva( parent, name, numFrames, cadRib )
		{			
			lista = Lista;

			Rm = RMIN;
			Ry = RMAY;
			M = theta;
			N1 = PHIMIN;
			N2 = PHIMAX;	
			iterIn = OPTTORUS;
			iterEx = OPTTORUS;
			factIn = (iterIn - 2)/ 360.0;
			factEx = (iterEx - 2)/ 360.0;
			color = Color;
		
			for(int i = 0, h = 0; i < iterEx; ++i)
			{
			  for(h = 0; h < iterIn; ++h)
			  {
			    _puntos[i][h] = new Matrix( 4, 1 );
			  }
			}
			
			matRib = mater;
			
			this -> recalcularValoresIniciales( );
			
		}
		
		~primToro()
		{
		  for(int i = 0, h = 0; i < iterEx; ++i)
			{
	  		for(h = 0; h < iterIn; ++h)
			  {
			    delete _puntos[i][h];
	  		}
			}
		}

	public slots:
    void generarCodGL( void ); // Listo
		void generarCodRIB( void ); // Listo
		void generarInterCreac( void );
		void actualizarValores( void );
	  void recalcularValoresIniciales( void );
    void validarNombre( void ) { if( ! dialogo ) emit _validarNombre( *nombre ); }
    void encabezadoXRIB( void );
    void genFrameXRib( void );
	protected:

	  dialogo_crear_toro * dialogo;

 		int iterIn, iterEx, indIn, indIn2, indEx;		
		
		float Rm, Ry, M, N1, N2, factIn, factEx;
		
		pMatrix _puntos[OPTTORUS][OPTTORUS];

};
#endif
