/***************************************************************************
    begin                : Mon Jul  2 23:00:58 COT 2001
    copyright            : (C) 2001 by Vctor Javier Cerquera, Alejandro Gutirrez
    email                : <javier@gaia.univalle.edu.co>
    											 <alejandro@calipso.com.co>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*	La clase primitiva contiene mtodos virtuales de lo que toda primitiva
		debe contener.  Es obligatorio para cualquier primitiva que se desee aadir
		al proyecto, heredar de esta clase base e implementar sus mtodos.

		A excepcin del constructor y del destructor, todos los otros mtodos de
		la clase son virtuales.
*/

#ifndef PRIMITIVA_H
#define PRIMITIVA_H

#include "defines.h"
#include "spinboxf.h"
#include "dialogo_crear_prisma.h"

typedef struct {

	uint number;
  Matrix * mat;

} primFrame;


class primitiva : public QObject
{
  Q_OBJECT

	public:

		primitiva( QObject* parent, const char* name, int numFrames, QString * cadRib )
							: QObject( parent, name )
		{
			actual = 0;
			desde = 0;
			hasta = numFrames - 1;
			
			animando = false;
			
			actualM = new primFrame;
			actualM -> mat = new Matrix( 4, 4 );
			actualM -> mat -> Unit();
			actualM -> number = 0;
			frames . append( actualM );
			
			preview = new Matrix( 4, 4 );
			preview -> Unit();
			keyF = false;
			selecc = true;
			nombre = new QString( name );
			cadRIB = cadRib;
			color = new QColor( blue );			
			matRib = 0;
			noGen = false;
			ultKeyFrame = 0;
    	frameXRibAct = 0;
    	matRib = new QString( );
		}
	
		~primitiva() {}
			
	public slots:
		virtual void generarCodGL( void ) {}
		virtual void encabezadoXRIB( void ) {}	
		virtual void generarCodRIB( void ) {}		
		virtual void generarInterCreac( void ) {}
    virtual	void actualizarValores( void ) {}
    virtual	void recalcularValoresIniciales( void ) {}
    virtual void validarNombre( void ) { }
		virtual void genFrameXRib( void ) {}

		void getNombre( void ) { emit _nombre( *nombre ); }
		
		void resetPreview( void ) { preview -> Unit( ); this -> recalcularValores( ); }
		
		void record( void )
		{
		  if ( actualM -> number != actual )
    	{
  	    primFrame *nuevo;  	
	    	nuevo = new primFrame;
      	nuevo -> mat = new Matrix( 4, 4 );
    	  nuevo -> number = actual;
  	    *( nuevo -> mat ) = *( actualM -> mat );
	
	      frames.append( nuevo );
      	this -> cambiarAFrame( actual );
      	
      	if(actualM -> number > ultKeyFrame)
      		ultKeyFrame = actualM -> number;
    	}
		}	
			
		void recalcularValores( void )
		{
			if( !animando )	
		    emit _actualizarValor( );
		}

		
		void getFrame( void )
		{
		  uint indice = 0, temp = 0;

			for (uint  n = 0 ; n < frames.count() ; n++ )
			{	
			  actualM = frames.at(n);
	
			  if ( actualM->number == actual )
			  {
			    indice = n;
			 		break;
			  }		
			
			  if ( actualM->number < actual && temp < actualM->number )
			  {
			    temp = actualM->number;
			    indice = n;
	  		} 		
			}
	
			actualM = frames.at( indice );
	
			this -> recalcularValores( );
		}
				
		void genRIBFrame( uint donde )
		{
		  uint tempo;

		  tempo = actual;
		  animando = true;

		  this -> cambiarAFrame( donde );
		  this -> generarCodRIB( );
		  this -> cambiarAFrame( tempo );
	
  		animando = false;

		  emit _animOk( );
		}

		void cambiarAFrame( int donde )
		{
			actual = donde;
		  getFrame( );
		}
		
		void material( QString * cual )
		{
		  if( selecc )
        matRib = cual;
    }
 		
		void addTransf( Matrix elem )
		{
		  if( selecc )
		  {
		    *( actualM -> mat ) = *( actualM -> mat ) * elem;
		    preview -> Unit( );
		  }
		}
		
		void addPreview( Matrix elem )
		{
		  if( selecc )
		    *preview = elem;
		  else
		  	preview -> Unit( );
  	
		  this -> recalcularValores( );
		}
		
		void addTransf( Matrix elem, QString * queNombre, uint cualFrame )
		{
		  if( *nombre == *queNombre )
		  {
		    this -> cambiarAFrame( cualFrame );
		    this -> record( );
		    *( actualM -> mat ) = elem;
		    preview -> Unit( );
		  }
		}

    void seleccionar( QString nomb, bool valor ) { selecc = ( nomb == *nombre )?valor:selecc; }

    void interpolar( void )
    {
      uint lim, max;
      primFrame * temp, * fin;
      Matrix * calc;

      fin = 0;
      calc = new Matrix( 4, 4 );
      if(selecc)
      {
	      max = frames.count();
  	
    	  for(uint i = 0, h; i < max; ++i)
      	{
        	calc -> Unit();
	        temp = frames.at(i);

  	      lim = ultKeyFrame;
    	
      	  if(temp -> number == ultKeyFrame)
	          continue;
  	      for(h = 0; h < max; ++h)
    	  	{
      		  if( frames.at(h) -> number > temp -> number && frames.at(h) -> number <= lim)
      		  {
      	  	  lim = frames.at(h) -> number;
      	    	fin = frames.at(h);
	      	  }
  	    	}
      	
    	 	  if( max > 0 )
     		  {
	      		*calc = *( fin -> mat ) - *( temp -> mat );
  	    		*calc /= ( fin -> number ) - ( temp -> number );

	    	  	for( h = temp -> number + 1; h < fin -> number; ++h )
  	    		{
    	  		  this -> cambiarAFrame( h );
				    	this -> record( );
			  	  	*( actualM -> mat ) = ( *calc );
			    		*( actualM -> mat ) *= ( h - temp -> number );
			    		*( actualM -> mat ) += *( temp -> mat );
	    	  	}      	
  	    	}
    	  }
      }
      emit _interpolarFin( );
    }

	protected:

		GLuint desde;
		GLuint hasta;
		GLuint actual;
    GLuint lista;		

    uint ultKeyFrame;
    uint frameXRibAct;

		bool keyF;
    bool selecc;
		bool noGen;
		bool animando;

    QColor *color;

	  QString * cadRIB;
		QString * nombre;
		QString * matRib;
		
		Matrix * preview;
		
		primFrame * actualM;
						
		QList<primFrame> frames;

	private:

	signals:

		void _objetoCreado( void );
		void _objetoNoCreado( void );
		void _actualizacionPrimOK( void );
		void _actualizarValor( void );
		void _RIBOk( void );
		void _animOk( void );
		void _encabezadoOK( void );
    void _frameOK( void );
    void _finXRIB( void );
		void _nombre( QString );
		void _validarNombre( QString );
		void _revisarNombre( const QString & );
	  void _interpolarFin( void );
};
#endif
